﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace StripControls
{
    public partial class Form2 : Form
    {
        // Объявили поле-ссылку на объект команды меню
        ToolStripMenuItem itemSelectedFont;
    
        public Form2()
        {
            InitializeComponent();
    
            // Заполнение коллекции объекта itemFont 
            // списком доступных шрифтов стиля "Обычный"
            Graphics gr = CreateGraphics(); // Ссылка на контекст устройства
            // Читаем все шрифты 
            FontFamily[] fonts = FontFamily.GetFamilies(gr);
    
            // Перебираем шрифты и формируем команды меню Font
            foreach (FontFamily font in fonts)
            {
                // Выбираем шрифты одного стиля "Обычный"
                if (font.IsStyleAvailable(FontStyle.Regular))
                {
                    // Добавляем в коллекцию команды Font
                    ToolStripMenuItem item = 
                        (ToolStripMenuItem)itemFont.DropDownItems.Add(font.Name);
    
                    // Если встретился текущий шрифт формы, 
                    // запоминаем эту команду и отмечаем флажком
                    if (font.Name == this.Font.Name)
                    {
                        itemSelectedFont = item;
                        itemSelectedFont.Checked = true;
                    }
    
                    // Следующую строчку введите вручную!!!
                    item.Click += new EventHandler(item_Click);
                }
            }
    
            gr.Dispose(); // Освобождаем ограниченный ресурс 
        }

        void item_Click(object sender, EventArgs e)
        {
            // Повышаем полномочия ссылки для адресации к выделенной команде 
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            itemSelectedFont.Checked = false;   // Сбрасываем предыдущий флажок
            item.Checked = true;    // Поднимаем новый флажок
            itemSelectedFont = item;// Запоминаем отмеченную команду
            panel.Invalidate();     // Перерисовываем панель с новым шрифтом
        }

        private void PanelOnPaint(object sender, PaintEventArgs e)
        {
            Graphics gr = e.Graphics;   // Получаем контекст устройства 
    
            // Создаем новый шрифт по выбранному в меню с заведомо
            // большим размером, чтобы потом подогнать уменьшением
            float emSize = 100.0F;
            System.Drawing.Font font = 
                new Font(itemSelectedFont.Text.Trim(), emSize); 
            String str = font.Name; // Строка для рисования
    
            // Подгоняем размер текстового блока под ширину формы 
            float widthText = gr.MeasureString(str, 
                font = new Font(itemSelectedFont.Text, emSize)).Width;
            while ((panel.ClientSize.Width - widthText) < 0.0F)
            {
                emSize -= 0.5F;
                widthText = gr.MeasureString(str, 
                    font = new Font(itemSelectedFont.Text, emSize)).Width;
            }
    
            // Настроим точку привязки текстового блока
            System.Drawing.StringFormat fmt = new StringFormat();
            fmt.Alignment = StringAlignment.Center; // По горизонтали
            fmt.LineAlignment = StringAlignment.Center;   // По вертикали
    
            // Настроим точку привязки клиентской области панели
            PointF point = new PointF(panel.ClientSize.Width / 2,
                panel.ClientSize.Height / 2);
    
            // Рисуем на панели название выбранного шрифта текущим цветом формы
            gr.DrawString(str, font, 
                new SolidBrush(this.ForeColor), point, fmt);
        }

        protected override void OnResize(EventArgs e)
        {
            panel.Invalidate();
            base.OnResize(e);
        }
    }
}