﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Resources;

namespace StripControls
{
    public partial class Form3 : Form
    {
        String originalTitle;   // Поле для хранения чистого заголовка
        String name;            // Поле для хранения имени загруженного рисунка
    
        public Form3()
        {
            InitializeComponent();
    
            LoadImages();// Присоединяем пиктограммы 
    
            CreateView();// Динамически заполняем меню View
    
            originalTitle = this.Text;// Сохраняем чистый заголовок
        }

        // Загрузка изображений в элементы меню
        void LoadImages()
        {
            // Присоединяем 2 пиктограммы из 
            // файла локальных ресурсов Form3.resx 
            ResourceManager resources =
                new ResourceManager(typeof(Form3));//или (this.GetType())
            itemCopy.Image = (Image)resources.GetObject("Copy");
            itemCut.Image = (Image)resources.GetObject("Cut");
    
            // Присоединяем пиктограмму из файла,
            // помеченного как Embedded Resource 
            itemPaste.Image = new Bitmap(this.GetType(),    // или (typeof(Form3)
                "Resources.Paste.bmp");
    
            ///////////////////////////////
            // Пиктограмма для Delete внедрена декларативно
            // через стандартный файл глобальных ресурсов,
            // поэтому для нее ничего не делаем 
            ///////////////////////////////
    
            // Присоединяем пиктограмму из пользовательского 
            // файла глобальных ресурсов MyResources.resx 
            itemOpen.Image = Resources.MyResources.Open;
    
            // Присоединяем пиктограмму непосредственно из файла рисунка
            itemSaveAs.Image = new Bitmap("Resources\\Save.bmp");
        }

        // Функция создания элементов меню View программным способом
        void CreateView()
        {
            // Имя элемента меню на верхнем уровне для возможности 
            // адресации в коллекции Items по его имени 
            itemView.Name = "View";
    
            ToolStripMenuItem item=new ToolStripMenuItem("&Normal");
            item.Tag = PictureBoxSizeMode.Normal;
            // Создайте вручную!!!!!!!
            item.Click += new EventHandler(ViewOnClick);
            itemView.DropDownItems.Add(item);// Добавили в коллекцию View 
            // Имя элемента меню View для возможности адресации
            // в коллекции DropDownItems по его имени
            item.Name = "Normal";
            // Отмечаем флажок, поскольку в начальном состоянии 
            // свойство SizeMode элемента PictureBox именно Normal
            item.Checked = true;
    
            item = new ToolStripMenuItem();
            item.Text = "&Center";
            item.Tag = PictureBoxSizeMode.CenterImage;
            item.Click += ViewOnClick;
            itemView.DropDownItems.Add(item);
            item.Name = "Center";
    
            item = new ToolStripMenuItem("&Stretch");
            item.Tag = PictureBoxSizeMode.StretchImage;
            item.Click += ViewOnClick;
            itemView.DropDownItems.Add(item);
            item.Name = "Stretch";
    
            item = new ToolStripMenuItem();
            item.Text = "&Zoom";
            item.Tag = PictureBoxSizeMode.Zoom;
            item.Click += ViewOnClick;
            itemView.DropDownItems.Add(item);
            item.Name = "Zoom";
    
            item = new ToolStripMenuItem("&AutoSize");
            item.Tag = PictureBoxSizeMode.AutoSize;
            item.Click += ViewOnClick;
            itemView.DropDownItems.Add(item);
            item.Name = "AutoSize";
        }

        void ViewOnClick(object sender, EventArgs e)
        {
            // Приводим ссылку на элемент меню View, сгенерировавший событие 
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
    
            // Распаковываем объект и используем сохраненную 
            // в нем информацию для управления контейнером рисунков
            pictureBox.SizeMode = (PictureBoxSizeMode)item.Tag;
    
            ////////////////////////////////////////////////////////
            // Управление флажками выполним в обработчике события
            // DropDownOpening, которое информирует о раскрытия узла View
            ////////////////////////////////////////////////////////
            //////////// Следующий код закомментирован /////////////
            ////////////      Жалко выбрасывать!!!     /////////////
            /*//////////////////////////////////////////////////////
            // Сбрасываем флажки всех элементов 
            foreach (ToolStripMenuItem it in itemView.DropDownItems)
                it.Checked = false;
            // Устанавливаем флажок на выделенном элементе,
            // адресуясь к коллекции по имени элемента
            ((ToolStripMenuItem)itemView.DropDownItems[item.Name]).Checked = true;
            //item.Checked = true;  // Альтернативный вариант
            //*/////////////////////////////////////////////////////
        }

        private void FileOnDropDown(object sender, EventArgs e)
        {
            // Сделать недоступным команду SaveAs, если контейнер пустой 
            itemSaveAs.Enabled = pictureBox.Image != null;
        }

        private void EditOnDropDown(object sender, EventArgs e)
        {
            /////////////////////////////////////////////////
            // Регулируем доступность команд Copy, Cut, Delete
            /////////////////////////////////////////////////
            // Нечего редактировать - контейнер пустой 
            itemCopy.Enabled = itemCut.Enabled = itemDelete.Enabled =
                pictureBox.Image != null;
    
            /////////////////////////////////////////////////
            // Регулируем доступность команды Paste
            /////////////////////////////////////////////////
            // Ивлекаем адрес системного буфера обмена, 
            // используя интерфейсную ссылку базового типа
            IDataObject data = Clipboard.GetDataObject();
            // Если системный буфер не содержит рисунков 
            // подходящего типа, то вставлять тоже нечего 
            itemPaste.Enabled = data.GetDataPresent(typeof(Bitmap))
                || data.GetDataPresent(typeof(System.Drawing.Imaging.Metafile));
        }

        private void ViewOnDropDown(object sender, EventArgs e)
        {
            // При раскрытии узла View перебираем коллекцию 
            // и управляем флажками с помощью состояния pictureBox
            foreach (ToolStripMenuItem item in itemView.DropDownItems)
            {
                item.Checked = 
                    (PictureBoxSizeMode)item.Tag == pictureBox.SizeMode;
            }
        }

        // Обработчик команды Open 
        OpenFileDialog openDialog; // Объявили ссылку на диалог
        bool openFlag = true;// Запускается в первый раз 
        //
        private void OpenOnClick(object sender, EventArgs e)
        {
            if (openFlag)
            {
                openFlag = false;   // Больше не создавать и не настраивать!
                // Создаем и настраиваем диалог
                openDialog = new OpenFileDialog();
                openDialog.Filter = "All allowable Image Files" +                               // 1)Заголовок
                    "|*.bmp;*.ico;*.gif;*.jpeg;*.jpg;*.jfif;*.png;*.tif;*.tiff;*.wmf;*.emf|" +  // 1)|Шаблон|
                    "Windows Bitmap (*.bmp)|*.bmp|" +                               // 2) Заголовок|Шаблон| 
                    "Graphics Interchange Format (*.gif)|*.gif|" +                  // 3) Заголовок|Шаблон|
                    "JPEG File Interchange Format (*.jpg)|*.jpeg;*.jpg;*.jfif|" +   // 4) Заголовок|Шаблон|
                    "Portable Network Graphics (*.png)|*.png|" +                    // 5) Заголовок|Шаблон|
                    "Tagged Image File Format (*.tif)|*.tif;*.tiff|" +              // 6) Заголовок|Шаблон|
                    "Windows Metafile (*.wmf)|*.wmf|" +                             // 7) Заголовок|Шаблон|
                    "Enhanced Metafile (*emf)|*.emf|" +                             // 8) Заголовок|Шаблон|
                    "All Files (*.*)|*.*";                                          // 9) Заголовок|Шаблон
    
                // Устанавливаем последний шаблон активным (по умолчанию 1!)
                String[] str = openDialog.Filter.Split('|');
                openDialog.FilterIndex = str.Length / 2;
            }
    
            // Защищенный код для перехвата системных исключений на 
            // случай попытки загрузки неподдерживаемого формата данных 
            if (openDialog.ShowDialog() == DialogResult.OK)
                try
                {
                    pictureBox.Image = Image.FromFile(openDialog.FileName);
    
                    // Выводим имя рисунка в заголовок окна
                    name = openDialog.FileName;
                    String[] str = name.Split('\\');
                    name = str[str.Length - 1].ToLower();
                    this.Text = "(" + name + ") " + originalTitle;
                }
                catch (Exception ext)
                {
                    MessageBox.Show(ext.Message, this.Text, 
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
        }
    
        // Обработчик команды Save As 
        SaveFileDialog saveDialog; // Объявили ссылку на диалог
        System.Drawing.Imaging.ImageFormat[] imageFormat;// Объявили ссылку на массив
        bool saveFlag = true;// Запускается в первый раз 
        //
        private void SaveAsOnClick(object sender, EventArgs e)
        {
            if (saveFlag)
            {
                saveFlag = false;   // Больше не создавать и не настраивать!
                // Создаем диалог
                saveDialog = new SaveFileDialog();
                // Настраиваем фильтр
                saveDialog.Filter = "Windows Bitmap (*.bmp)|*.bmp|" + 
                    "Graphics Interchange Format (*.gif)|*.gif|" + 
                    "JPEG File Interchange Format (*.jpg)|*.jpeg;*.jpg;*.jfif|" + 
                    "Portable Network Graphics (*.png)|*.png|" + 
                    "Tagged Image File Format (*.tif)|*.tif;*.tiff";
    
                // Инициализируем массив допустимыми форматами рисунков
                imageFormat = new System.Drawing.Imaging.ImageFormat[] {
                    System.Drawing.Imaging.ImageFormat.Bmp,
                    System.Drawing.Imaging.ImageFormat.Gif,
                    System.Drawing.Imaging.ImageFormat.Jpeg,
                    System.Drawing.Imaging.ImageFormat.Png,
                    System.Drawing.Imaging.ImageFormat.Tiff
                };
            }
    
            // Заполняем поле именем загруженного рисунка без расширения
            saveDialog.FileName = name.Substring(0, name.LastIndexOf('.'));
    
            // Защищенный код для перехвата системных 
            // исключений на случай сбоя записи
            if (saveDialog.ShowDialog() == DialogResult.OK)
                try
                {
                    // Пытаемся сохранить в формате текущего фильтра
                    pictureBox.Image.Save(saveDialog.FileName,
                        imageFormat[saveDialog.FilterIndex - 1]);
           
                    // Обновляем контейнер новым форматом - ???????????????
                    //pictureBox.Image = Image.FromFile(saveDialog.FileName);
    
                    // Выводим в заголовок окна имя файла с новым расширением
                    name = saveDialog.FileName;
                    String[] str = name.Split('\\');
                    name = str[str.Length - 1].ToLower();
                    this.Text = "(" + name + ") " + originalTitle;
                }
                catch (Exception ext)
                {
                    MessageBox.Show(ext.Message, this.Text,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
        }

        private void ExitOnClick(object sender, EventArgs e)
        {
            this.Close();
        }

        private void CutOnClick(object sender, EventArgs e)
        {
            ////////////////////////////////////////////////
            // Используем уже готовые обработчики!!!
            ////////////////////////////////////////////////
            // Копируем в системный буфер обмена
            CopyOnClick(sender, e);
            // Обнуляем контейнер рисунка
            DeleteOnClick(sender, e);
        }

        private void CopyOnClick(object sender, EventArgs e)
        {
            // Заполняем системный буфер обмена рисунком контейнера
            // Если при раскрытии узла Edit текущий 
            // рисунок есть, то команда Copy доступна
            Clipboard.SetDataObject(pictureBox.Image, true);
        }

        private void PasteOnClick(object sender, EventArgs e)
        {
            // Подключаемся к системному буферу обмена 
            IDataObject data = Clipboard.GetDataObject();
    
            // Распознаем тип изображения в буфере обмена и читаем данные
            Type typeMeta = typeof(System.Drawing.Imaging.Metafile);
            if (data.GetDataPresent(typeMeta))
                pictureBox.Image = (Image)data.GetData(typeMeta);
            else
                pictureBox.Image = (Image)data.GetData(typeof(Bitmap));
        }

        private void DeleteOnClick(object sender, EventArgs e)
        {
            pictureBox.Image = null;
        }
    }
}