﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Reflection;
    
namespace StripControls
{
    public partial class Form5 : Form
    {
        // Поле для сохранения выбранного элемента меню
        ToolStripMenuItem itemChecked = new ToolStripMenuItem();
    
        // Конструктор
        public Form5()
        {
            InitializeComponent();
    
            splitContainer.Panel1.BackColor = Color.White;// Начальный цвет верхней панели
    
            // Разделение цветов на алфавитные группы, представленные подменю
            //             A, B, C, D, E, F, G, H, I, J, K, L, M,           
            int[] iMenu ={ 0, 0, 0, 1, 2, 2, 2, 2, 2, 2, 2, 3, 4,  
                5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 6, 6 };
            //  N, O, P, Q, R, S, T, U, V, W, X, Y, Z
    
            // Создание контекстного меню и присоединение к объекту формы
            System.Windows.Forms.ContextMenuStrip menu =
                new System.Windows.Forms.ContextMenuStrip();
            // Будет вызываться для верхней панели
            splitContainer.Panel1.ContextMenuStrip = menu;
    
            // Формируем пункты подменю в контекстном меню
            for (int i = 0; i <= 6; i++)
            {
                // Индекс первого вхождения образца соответствует ASCII буквы 
                char first = Convert.ToChar(Array.IndexOf(iMenu, i) + 'A');   // Позиция в массиве
                // Индекс последнего вхождения образца соответствует ASCII латинской буквы 
                char last = Convert.ToChar(Array.LastIndexOf(iMenu, i) + 'A');// Позиция в массиве
    
                // Создали, настроили и добавили в коллекцию элемент меню
                ToolStripMenuItem item = new ToolStripMenuItem();
                item.Text = String.Format("Цвета от {0} до {1}", first, last);
                menu.Items.Add(item);
                // Включили в подменю столбец с флажками в 
                // дополнение к столбцу с изображениями
                ((ToolStripDropDownMenu)item.DropDown).ShowCheckMargin = true;
            }
    
            // Применяем рефлексию к структуре, представляющей цвета
            PropertyInfo[] api = typeof(Color).GetProperties();
            // Перебор цветов
            foreach (PropertyInfo pi in api)
            {
                if (pi.CanRead && pi.PropertyType == typeof(Color)
                    && pi.Name!="Transparent")
                {
                    // Извлекаем очередной цвет
                    Color color = (Color)pi.GetValue(null, null);
    
                    // Создаем очередной элемент подменю
                    ToolStripMenuItem item = new ToolStripMenuItem();
                    // По букве цвета определяем подменю, в котором разместим цвет
                    int i = iMenu[color.Name[0] - 'A'];// Любой букве - своя позиция
                    ((ToolStripMenuItem)menu.Items[i]).DropDownItems.Add(item);
    
                    // Продолжаем настраивать команду
                    item.Text = InsertSpaceOut(color.Name);// Пробелы перед заглавными буквами цвета
                    item.Name = color.Name;// Сохраняем название цвета для обработчика ColorOnClick
                    item.Image = CreateBitmap(color);// Тонкой ссылке присваиваем толстый объект
                    item.Click += new EventHandler(ColorOnClick);// Регистрация обработчика
                    if (color.Equals(this.BackColor))// Отметить флажком текущий цвет и сохранить ссылку
                        (itemChecked = item).Checked = true;
                }
            }
        }
    
        void ColorOnClick(object sender, EventArgs e)
        {
            // Для выбранного устанавливаем новый флажок
            // и меняем цвет клиентской области формы
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            itemChecked.Checked = false;// Сбрасываем старое
            (itemChecked = item).Checked = true;// Устанавливаем флажок и запоминаем элемент
            splitContainer.Panel1.BackColor = Color.FromName(item.Name);// Красим верхнюю панель
        }
    
        Bitmap CreateBitmap(Color color)
        {
            // Генерируем изображение в виде цветного прямоугольника
            Bitmap bm = new Bitmap(16, 16);
            Graphics gr = Graphics.FromImage(bm);
            gr.FillRectangle(new SolidBrush(color), 0, 0, 16, 16);
            gr.Dispose();   // Освобождаем контекст устройства
            return bm;
        }
    
        string InsertSpaceOut(string str)
        {
            // Вставляем пробелы перед заглавными буквами названия цвета
            for (int i = str.Length - 1; i > 1; i--)
                if (Char.IsUpper(str[i]))
                    str = str.Insert(i, " ");
            return str;
        }
    
        private void splitContainer1_Panel2_BindingContextChanged(object sender, EventArgs e)
        {
            if (radioButton1.Checked)
                splitContainer1.Panel2.ContextMenuStrip = contextMenuStrip1;
            else if (radioButton2.Checked)
                splitContainer1.Panel2.ContextMenuStrip = contextMenuStrip2;
            else
                splitContainer1.Panel2.ContextMenuStrip = contextMenuStrip3;
        }

        private void radioButton_CheckedChanged(object sender, EventArgs e)
        {
                if (radioButton1.Checked)
                    splitContainer1.Panel2.ContextMenuStrip = contextMenuStrip1;
                else if (radioButton2.Checked)
                    splitContainer1.Panel2.ContextMenuStrip = contextMenuStrip2;
                else
                    splitContainer1.Panel2.ContextMenuStrip = contextMenuStrip3;
        }
    }
}